# Generated by Django 3.2.5 on 2023-03-28 08:50

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("accounts", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Notebook",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("file_updated_at", models.DateTimeField(blank=True)),
                ("title", models.CharField(max_length=512)),
                ("slug", models.CharField(blank=True, max_length=512)),
                ("path", models.CharField(max_length=1024)),
                (
                    "image_path",
                    models.CharField(blank=True, default="", max_length=1024),
                ),
                ("params", models.TextField(blank=True)),
                ("state", models.CharField(blank=True, max_length=128)),
                ("default_view_path", models.CharField(blank=True, max_length=1024)),
                ("output", models.CharField(blank=True, max_length=128)),
                ("format", models.CharField(blank=True, max_length=1024)),
                ("schedule", models.CharField(blank=True, max_length=128)),
                ("notify", models.TextField(blank=True)),
                ("errors", models.TextField(blank=True)),
                (
                    "created_by",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "hosted_on",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="accounts.site"
                    ),
                ),
            ],
        ),
    ]
